#include "lib.lua"

local substeps = FGetInt("savegame.mod.substeps", 8)
local laserDamage = FGetFloat("savegame.mod.holesize", 0.25)
local laserRange = FGetFloat("savegame.mod.range", 8)

local laserSound = LoadLoop("snd/laser.ogg")
local snd5 = LoadLoop("snd/electricity.ogg")
local laserSprite = LoadSprite("gfx/laser.png")
laserchargespeed = 1.1

local laserLong
lasercuttergroup = 3

function lasercutter_init()
	RegisterTool("lasercutter", "Laser Cutter", "MOD/vox/lasercutter.vox", lasercuttergroup)
	SetBool("game.tool.lasercutter.enabled", true)
	-- SetFloat("game.tool.lasercutter.ammo", 101)
	SetFloat("game.tool.lasercutter.ammo", 10000) -- Hides ammo count

end

function GetAimPos(range, ct)
	local forwardPos = TransformToParentPoint(ct, Vec(0, 0, -range))
    local direction = VecSub(forwardPos, ct.pos)
    local distance = VecLength(direction)
	local direction = VecNormalize(direction)
	-- QueryRejectBody(GetToolBody())
	local hit, hitDistance = QueryRaycast(ct.pos, direction, distance)
	if hit then
		forwardPos = TransformToParentPoint(ct, Vec(0, 0, -hitDistance))
	end
	return forwardPos, hit
end

local prevpos = nil
local prevct = nil
function ShootLaser(range)
	local range = range
	local anyhit = false
	for s=0, substeps+1 do
		local ct = GetCameraTransform()
		local lt = ct
		local tt = s/substeps
		if prevct ~= nil then
			lt = Transform(VecLerp(ct.pos,prevct.pos,tt),QuatSlerp(ct.rot,prevct.rot,tt))
		elseif s>0 then break end

		local aimpos, hit = GetAimPos(range, lt)

		-- if not hit then break end

		local steps = VecLength(VecSub(prevpos,aimpos))/laserDamage*1.25
		if steps > 50 then
			prevpos = nil
			prevct = nil
		end
		if prevpos ~= nil and steps>1 then
			for i=1,steps do
				local t = i/steps
				QueueHole(VecLerp(prevpos,aimpos,t), laserDamage, laserDamage, laserDamage, true)
			end
		else
			QueueHole(aimpos, laserDamage, laserDamage, laserDamage, true)
		end
		if hit then 
			prevpos = aimpos
			anyhit = true 
		end
		if s==0 and anyhit then
			PlayLoop(laserSound, ct.pos, range/50-0.4)

			-- VFX
			SpawnParticle("darksmoke", aimpos, Vec(0, 1.0+math.random(1,10)*0.1, 0), 0.1, 0.3)
			SpawnParticle("smoke", aimpos, Vec(0, 1.0+math.random(1,10)*0.1, 0), 0.3, 1.5)
			-- SpawnFire(aimpos)
			local t = GetCameraTransform()
			local tb = GetToolBody()
			local tbt = GetBodyTransform(tb)
			local pos = tbt.pos
			local dir = TransformToParentVec(tbt, Vec(-0.02, 0.02, -1))
			local hit, dist, normal, shape = QueryRaycast(pos, dir, 100, -1)
			local p = TransformToParentPoint(t, Vec(.35, -.3, -1.55))
			local q = TransformToParentPoint(t, Vec(.35, -.3, -1.30))
			local pp = TransformToParentPoint(t, Vec(.45, -.5, -1.65))
			local qq = TransformToParentPoint(t, Vec(.45, -.5, -4))
			local d = TransformToParentVec(t, Vec(-0.07, .06, -1))

			if hit then --Plasma Cutter Spark Effects
				local HitPoint = VecAdd(pos, VecScale(dir, dist))
				local vel1 = math.random(10,70)*0.1
				local vel2 = math.random(0,60)*0.1
				local vel3 = math.random(10,30)*0.1
				local vel4 = math.random(0,20)*0.1
				local u = Vec((vel1-4), (vel1 * .5), (vel2-3))
				local v = Vec((vel3-2), (vel3 * .5), (vel4-1))
				local w = Vec((vel2-3), (vel1 * .5), (vel1-4))
				local x = Vec((vel4-1), (vel3 * .5), (vel3-2))
				local life = 0.1 * math.random(3, 5)
				local life1 = life * 5
				local life2 = life * 4

				ParticleReset()
				ParticleEmissive(1, 0, "easein")
				ParticleGravity(-10)
				ParticleRadius(math.random(1, 2)*.01, 0.0, "smooth") 
				ParticleColor(.7,.7,1, 1,.2,0)
				ParticleTile(4)
				ParticleDrag(0, 0.7)
				ParticleCollide(0, 1, "easeout")
				SpawnParticle(aimpos, u, life1) -- white/violet to red/orange

				ParticleReset()
				ParticleEmissive(1, 0, "easein")
				ParticleGravity(-5)
				ParticleRadius(math.random(2, 3)*.01, 0.0, "easein")
				ParticleColor(1,.8,0.6, 1,.2,0) 
				ParticleTile(4)
				ParticleDrag(0, 0.7)
				ParticleCollide(0, 1, "easeout")
				SpawnParticle(aimpos, v, life2) -- white/yellow to red/orange

				ParticleReset()
				ParticleEmissive(1, 0, "easein")
				ParticleGravity(-7.5)
				ParticleRadius(math.random(2, 3)*.01, 0.0, "smooth")
				ParticleColor(.8,.8,1, 1,.5,0)
				ParticleTile(4)
				ParticleDrag(0, 0.7)
				ParticleCollide(0, 1, "easeout")
				SpawnParticle(aimpos, w, life1*.5) -- white/violet to orange

				ParticleReset()
				ParticleEmissive(1, 0, "easein")
				ParticleGravity(-20)
				ParticleRadius(math.random(1, 2)*.01, 0, "easein")
				ParticleColor(1,.8,0.6, 1,.4,0)
				ParticleTile(4)
				ParticleDrag(0, 0.7)
				ParticleCollide(0, 1, "easeout")
				ParticleSticky(0.3, 1)
				SpawnParticle(HitPoint, x, life2*5) -- white/yellow to orange
				-- Play electrical spark sound
				PlayLoop(snd5, HitPoint, (math.random(7,  10)*0.1))
				PointLight(aimpos, .8, .8, 1, 1)
			end
		end
	end


	MakeQueue()

	prevct = GetCameraTransform()

	if not anyhit then
		prevpos = nil
		prevct = nil
	end

end

-- Shamelessly copied from my grapple mod
local hookCost = 1
local maxMeter = 10
SetFloat("game.tool.lasercutter.meter",maxMeter)
function lasercutter_draw()
	if FGetBool("level.unlimitedammo") then SetFloat("game.tool.lasercutter.meter", 100) return end

	if GetString("game.player.tool") == "lasercutter" and GetPlayerVehicle() == 0 then
		UiPush()
		local a=1
		if FGetFloat("game.tool.lasercutter.meter") < hookCost then a=0.5 end
		UiTranslate(UiWidth()/2-78, UiHeight()-60)
		progressBar(78*2, 20, FGetFloat("game.tool.lasercutter.meter")/maxMeter, a)
		UiTranslate(78*2 * hookCost/maxMeter)
		UiImage("MOD/ui/line.png")
		UiPop()
	end
end

local time = 0
local shooting = false
function lasercutter_tick(dt)
	if GetFloat("game.tool.lasercutter.meter") > maxMeter then SetFloat("game.tool.lasercutter.meter", maxMeter) end


	if GetString("game.player.tool") == "lasercutter" and GetPlayerVehicle() == 0 then


		if (InputDown("usetool") or (InputDown("rmb") and GetPlayerGrabBody() == 0)) and (GetFloat("game.tool.lasercutter.meter") > hookCost or shooting) then
			local range = laserRange / 1.7
			shooting = true
			local aimpos, hit = GetAimPos(range, GetCameraTransform())

			if GetFloat("game.tool.lasercutter.meter") < 0 then
				SetFloat("game.tool.lasercutter.meter", 0)
				shooting = false
			end

			if InputDown("usetool") then
				if (InputDown("rmb") and GetPlayerGrabBody() == 0) then
					DrawSpiral(toolPos, aimpos, 0.04, -4,     0.1, 1.0, 0.1)
					DrawSpiral(toolPos, aimpos, 0.06, -8,     1.0, 0.1, 0.1)
					DrawSpiral(toolPos, aimpos, 0.02, -2,     0.1, 0.1, 1.0)
					DrawLine(toolPos, aimpos, 1, 1, 1)

					laserDamage = 0.8
				else
					DrawLine(toolPos, aimpos, 1.0, 0.1, 0.1)
					DrawSpiral(toolPos, aimpos, 0.05, -8,     1.0, 0.1, 0.1)
					laserDamage = 0.35
				end
			else
				DrawLine(toolPos, aimpos, 0.1, 0.8, 0.8)
				DrawSpiral(toolPos, aimpos, 0.01, -4,     0.1, 0.8, 0.8)
				laserDamage = 0.15
			end

			SetFloat("game.tool.lasercutter.meter",GetFloat("game.tool.lasercutter.meter")-(laserDamage-0.1)*24*dt)

			if InputDown("usetool") then range = laserRange*5 + 80 - laserRange/2 end
			if (InputDown("rmb") and GetPlayerGrabBody() == 0) then range = range * 1.7 end
			ShootLaser(range)

		else
			prevpos = nil
			prevct = nil
			shooting = false
			SetFloat("game.tool.lasercutter.meter",GetFloat("game.tool.lasercutter.meter")+dt*laserchargespeed)

		end

		local b = GetToolBody()
		if b ~= 0 then
			local heightOffset = InputDown("ctrl") and 0 or 0.1
			local offset = Transform(Vec(0.1, -heightOffset, -0.2))
			SetToolTransform(offset)
			toolTrans = GetBodyTransform(b)
			toolPos = TransformToParentPoint(toolTrans, Vec(0.55, -0.55, -1.8))

			if InputDown("lmb") then
				PointLight(toolPos, 1, 0.1, 0.1, 1)
			end
		end
	else
		prevpos = nil
		prevct = nil
		shooting = false
		SetFloat("game.tool.lasercutter.meter",GetFloat("game.tool.lasercutter.meter")+dt*laserchargespeed)

	end

	time = time + dt

end

local holequeue = {}
function QueueHole(pos, s1, s2, s3, snd)
	holequeue[#holequeue+1] = {pos, s1, s2, s3, snd}
end
function MakeQueue()
	for i=1, #holequeue do
		local a = holequeue[i]
		MakeHole(a[1],a[2],a[3],a[4],a[5])
	end
	holequeue = {}
end

function DrawSpiral(s,e,v, speed, r,g,b)
	local steps = 100 * VecLength(VecSub(s,e))
	local ts = 1/steps
	local quat = QuatLookAt(s,e)
	quat = QuatRotateQuat(quat, QuatEuler(0, 0, 360*speed*time))
	local len = v
	for i=1, steps do
		len = len * 0.97
		local t = ts*i
		local ss = VecLerp(s,e,t-ts)
		local se = VecLerp(s,e,t)
		ss = VecAdd(ss, QuatRotateVec(quat,Vec(len, 0, 0)))
		quat = QuatRotateQuat(quat, QuatEuler(0, 0, 15))
		se = VecAdd(se, QuatRotateVec(quat,Vec(len, 0, 0)))
		DrawLine(ss, se, r,g,b)
		if len < 0.001 then break end
	end
end

function lasercutter_update() end