-- Math
function clampZero (val)
	if val > 0 then return val
	else return 0 end
end

function abs(val)
	if val >= 0 then return val
	else return -val end
end

function round(n)
    if n%1 > 0.5 then return n-n%1+1
    else return n-n%1 end
end
function floor(n) return n-n%1 end
function ceil(n) return n-n%1+1 end


function VecNormalize(vec)
	local vl = VecLength(vec)
	if vl == 0 then return Vec(0,0,0) end --Prevent / 0
	return VecScale(vec, 1/vl)
end

height = 1

function GetPlayerGrounded()
	local pp = GetPlayerPos()
	local hit, d, normal, shape = QueryRaycast(pp,Vec(0,-1,0), height, 0)

	local impactPoint = VecAdd(pp,VecScale(Vec(0,-1,0),d))
	groundMaterialType = GetShapeMaterialAtPosition(shape, VecAdd(impactPoint,Vec(0,-0.1,0)))
	return hit
end

function GetPlayerNearGround()
	local pp = GetPlayerPos()
	local hit, d = QueryRaycast(pp,Vec(0,-1,0), height*1.5, 0)
	return hit
end


-- Adds force in a radius
function AddForce(point, radius, impulse, scaling)
	local size = Vec(radius, radius, radius)
	local list = QueryAabbBodies(VecSub(point,size), VecAdd(point,size))
	for i=1, #list do
		local body = list[i]
		local dist = VecLength(VecSub(GetBodyCenterOfMass(body), point)) / radius
		if dist <= 1 then
			ApplyBodyImpulse(body, point, VecScale(impulse,(1-dist/2)))
		end
	end
end

-- UI

function progressBar(w, h, t,a )
	UiPush()
		UiAlign("left top")
		UiColor(0, 0, 0, 0.5*a)
		UiImageBox("ui/common/box-solid-10.png", w, h, 6, 6)
		if t > 0 then
			UiTranslate(2, 2)
			w = (w-4)*t
			if w < 12 then w = 12 end
			h = h-4
			UiColor(1,1,1,1*a)
			UiImageBox("ui/common/box-solid-6.png", w, h, 6, 6)
		end
	UiPop()
end


function optionsSlider(val, min, max, step)
	accent()
	UiPush()
		UiTranslate(0, -8)
		val = (val-min) / (max-min)
		--val = math.sqrt(val)
		local w = panelwidth
		local done = false
		accent()
		UiRect(w, 5)
		UiColor(0.5, 0.5, 0.5)
		UiRect(w*(1-val),5)
		UiAlign("center middle")
		UiTranslate(-panelwidth, 2)
		accent()
		val, done = UiSlider("ui/common/dot.png", "x", val*w, 0, w) / w
		--val = slider_round((val*val*(max-min)+min))
		val = slider_round((val*(max-min)+min), step)
	UiPop()
	return val, done
end


function optionsSliderExponential(val, min, max, step, exponent)
	accent()
	UiPush()
		UiTranslate(0, -8)
		val = (val-min) / (max-min)
		val = val^(1/exponent)
		local w = panelwidth
		local done = false
		accent()
		UiRect(w, 5)
		UiColor(0.5, 0.5, 0.5)
		UiRect(w*(1-val),5)
		UiAlign("center middle")
		UiTranslate(-panelwidth, 2)
		accent()
		val, done = UiSlider("ui/common/dot.png", "x", val*w, 0, w) / w
		val = slider_round((val^exponent*(max-min)+min), step)
		--val = slider_round((val*(max-min)+min), step)
	UiPop()
	return val, done
end

function optionsLine(val)
	accent()
	UiPush()
		UiTranslate(0, -8)

		local w = panelwidth
		local done = false
		if val then accent() else UiColor(0.5, 0.5, 0.5) end
		UiRect(w, 5)
		UiAlign("center middle")
		UiTranslate(-panelwidth, 2)
		accent()

	UiPop()
	return val, done
end

function slider_round(n1, n2)
    return math.floor((n1+0.5*n2)/n2) * n2
end

function accent()
	UiColor(0.99, 0.73, 0.125)
end

-- Mod Compatibility

function IsModEnabled() -- Used for compatibility between my mods
    return not HasKey("game.incompatible."..modname)
end

function GetSuccessor() -- Used for compatibility between my mods
    return GetString("game.incompatible."..modname)
end

function DisableMod(modtodisable)
    SetString("game.incompatible."..modtodisable, modname)
end

function AlertDisabled()
	DebugPrint("Mod \""..modname.."\" disabled for compatiblity. (Superceded by "..GetSuccessor()..")")
end

-- "Fixed" getters that actually respect default values. Returns nil if no default and key is nonexistant
function FGetFloat(key, default)
	if HasKey(key) then return GetFloat(key) else return default end
end
function FGetInt(key, default)
	if HasKey(key) then return GetInt(key) else return default end
end
function FGetString(key, default)
	if HasKey(key) then return GetString(key) else return default end
end
function FGetBool(key, default)
	if HasKey(key) then return GetBool(key) else return default end
end

